
# q-expansion of newform 5175.2.a.bf, downloaded from the LMFDB on 28 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5175
weight = 2
poly_data = [-3, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, -1], [0, 0], [0, 0], [-1, 1], [-1, -1], [3, -1], [-1, 0], [0, -2], [1, 0], [4, -2], [-3, 3], [5, 2], [-3, 2], [1, -2], [-3, 2], [-3, 5], [-3, 1], [-11, 3], [10, -1], [-7, 3], [-3, -4], [5, -2], [-7, 4], [-7, -2], [-3, 1], [-6, 3], [2, 2], [-9, 0], [-1, 0], [-6, 5], [6, -4], [3, 6], [-2, -4], [-13, -3], [-1, 0], [3, -8], [-20, 0], [7, 0], [-6, -3], [3, -3], [11, 5], [0, 2], [-3, 8], [5, 8], [-12, -6], [-2, -6], [22, -2], [-2, 3], [-3, -2], [-5, -10], [-11, 5], [2, 2], [16, 3], [-3, 3], [14, 7], [10, -15], [-24, -2], [-7, 2], [-20, -1], [-29, 2], [19, -10], [5, -14], [2, 0], [-13, 10], [13, 0], [17, -7], [-16, 13], [-5, 16], [10, -8], [-9, 10], [-1, 1], [9, -6], [17, -2], [19, -2], [4, 12], [-9, 9], [17, 1], [-32, 4], [15, -3], [-16, -6], [-28, -3], [6, -17], [9, -18], [9, -10], [-25, 2], [-2, -16], [7, -3], [-6, 8], [-17, 5], [-3, -15], [-12, 10], [16, -10], [24, -2], [27, -1], [-25, 14], [7, -12], [5, -7], [-8, -14], [5, 0], [-4, 10], [3, 14], [-16, 10], [-22, -5], [-27, 0], [-21, 2], [1, -21], [-10, -11], [15, -20], [-21, -4], [14, -12], [14, -9], [-2, 1], [-17, 11], [-9, 5], [2, 11], [-19, 7], [-11, 11], [30, -4], [-23, 0], [12, 7], [3, 13], [24, 0], [8, 19], [-15, -12], [1, -6], [-9, 2], [-14, 6], [-9, -12], [-14, 12], [-39, 10], [-9, 7], [-26, 1], [-5, -15], [17, -21], [-11, -4], [6, -13], [36, 0], [13, 4], [23, -4], [15, -22], [-17, 2], [-5, 20], [-23, 21], [-31, -6], [-2, -10], [-34, 9], [10, 8], [9, -19], [-6, -5], [15, -2], [17, -23], [-4, -7], [6, -8], [-9, 21], [18, 6], [3, -8], [22, -18], [-3, 11], [-6, -18], [-20, 19], [-22, 19], [11, -6], [35, -10], [-8, 2], [18, 2], [33, 3], [-14, 10], [-19, 7], [19, -14], [-18, 10], [23, -23], [-46, 8], [24, 0], [-22, 13], [41, -1], [0, 24], [47, 1], [7, -4], [-12, 7], [48, -5], [-7, 14], [24, -28], [-29, 13], [15, -11], [5, -8], [36, -23], [1, -16], [44, -6], [-10, 24], [39, -11], [7, -26], [20, 6], [45, -12], [37, -18], [37, -4], [2, -4], [-6, 7], [-19, 31], [-33, 21], [-27, 28], [32, 10], [-25, -11], [-10, 22], [-25, -4], [41, 1], [-9, -18], [-13, 6], [-36, 11], [34, -13], [-4, 12], [12, 4], [-46, -3], [-1, 25], [-25, 12], [-39, 0], [3, 17], [21, -31], [4, 11], [42, -8], [-5, 20], [-23, 18], [33, -19], [-12, -18], [-14, -14], [-22, -13], [-49, 12], [57, -3], [9, -25], [10, -8], [-13, -2], [11, -2], [-3, -20], [29, 0], [-38, 12], [-49, 2], [45, -3], [-35, 14], [36, -5], [-5, -4], [18, 24], [-3, 9], [18, -6], [-39, 23], [34, -22], [35, 10], [-28, -16], [-49, 5], [18, -2], [49, 7], [-59, 13], [-27, 8], [30, -26], [-30, 0], [3, -21], [18, 19], [27, 12], [-14, -3], [39, -11], [46, 14], [45, -17], [-20, -20], [-16, 32], [-26, -20], [49, -19], [53, 4], [8, -19], [21, -2], [-49, 21], [25, 6], [-18, -19], [-13, 0], [-8, -32], [-8, -15], [13, -19], [-24, 12], [-34, 11], [30, -30], [10, 11], [30, -24], [-19, 10], [-26, 18], [-37, -16], [-38, -6], [11, -14], [-11, 20], [36, 3], [4, -23], [18, 21], [-4, 39], [-53, 11], [-3, -31], [20, 8], [65, -14], [-4, -23], [-2, -18], [-26, 32], [-29, 28], [15, -15], [9, 30], [19, 18], [17, 7], [-7, 36], [-11, 18], [-53, 14], [-53, 9], [27, 13], [-54, 12], [-6, -28], [25, -7], [22, 8], [41, -5], [48, -18], [-37, 1], [20, 26], [-30, 10], [-43, -10], [-9, -12], [-42, -9], [-13, -14], [-44, 14], [-20, -8], [-3, -11], [6, -8], [2, 0], [6, -12], [49, -12], [-21, 23], [-12, 23], [21, 15], [-15, -13], [33, -12], [0, -31], [19, -38], [-30, -8], [13, -32], [-43, 23], [51, 17], [-7, 5], [-54, -4], [44, 7], [74, -6], [-28, 5], [51, 12], [-35, 2], [34, 7], [-24, 12], [-39, -7], [-22, 3], [24, -15], [11, -19], [56, 16], [-25, -17], [39, -10], [-17, 10], [38, -20], [-10, 7], [-12, 2], [33, 0], [-37, 7], [49, -14], [-55, 3], [39, -26], [-39, 21], [-35, 6], [10, 15], [8, -18], [-71, -4], [-53, -16], [-61, 21], [4, -7], [-16, 46], [6, -28], [87, -3], [-1, 34], [-89, 1], [-35, 39], [61, 2], [14, -28], [55, -7], [-23, -29], [-79, 0], [-21, 33], [30, -28], [-56, 3], [-6, -9], [0, 29], [-33, 44], [-20, -6], [41, 10], [50, -8], [-25, 43], [16, -2], [36, -38], [5, 14], [33, 22], [30, 22], [56, 12], [-66, 0], [81, -14], [9, -36], [-49, 41], [29, -26], [51, 20], [-15, 43], [-5, 10], [-52, -4], [42, 4], [-85, 0], [54, -18], [23, 31], [8, 31], [-9, 15], [-52, 6], [16, -1], [-10, 35], [17, -4], [-46, 3], [11, -15], [-5, 38], [16, -42], [-5, -20], [-46, 36], [42, -24], [65, -10], [-41, 36], [-22, 0], [-41, -15], [3, 21], [-38, 10], [62, 8]]
